@@ -1,4 +1,4 @@
 pub(crate) mod csv;
-pub(crate) mod memory;
+//pub(crate) mod memory;
 pub(crate) mod sheet;
 pub(crate) mod sled;
